//*********************************************************************
//* SPI Test for a SPI connected SRam on a Raspberry
//*********************************************************************
//* Description :
//* This Program should communicate with SPI to a static ram
//* The RAM has a 32,768 x 8-bit Organization and 32-Byte Pages.
//*
//********************************************************************
//* Used unit test_spi from http://wiki.freepascal.org/Raspberry_Pi_-_SPI
//* Written on a Raspberry 3 (stretch)
//* Lazarus : v1.6.2
//* Autor   : M.\\/eber  (only made the IDE)
//* Version : 1.00  Date : 03.11.2017
//***********************************************************************
//* How to use :
//* Use a SRAM 23K256 (Microchip).
//* Connect Pin 1 1/CS (Chip Select Input) to GPIO8 (CE0) oder GPIO7 (CE1)
//* Connect Pin 2 SO (Serial Data Output) to GPIO 9 (MISO)
//* Do not connect Pin 3 NC
//* Connect Pin 4 VSS to Ground
//* Connect Pin 5 SI (Serial Data Input) to GPIO 10 (MOSI)
//* Connect Pin 6 SCK (Serial Clock Input) to GPIO 11 (SCLK)
//* Connect Pin 7 1/Hold (Hold Input) to 3.3 VDC
//* Connect Pin 8 VCC () to 3.3 VDC

//*************************************************************************
//* Additional Info :
//* Datasheet : http://ww1.microchip.com/downloads/en/DeviceDoc/22100F.pdf


unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  Spin, Arrow, EditBtn, test_spi;

type

  { TForm1 }

  TForm1 = class(TForm)
    btnDataIn: TButton;
    btnDataOut: TButton;
    btnSendReceive: TButton;
    btnSendOnly: TButton;
    btnSingleByte: TButton;
    btnPageWrite: TButton;
    btnByteWrite: TButton;
    btnByteRead: TButton;
    btnSetStatusSequential: TButton;
    btnReadStatus: TButton;
    btnSetStatusPage: TButton;
    btnSetStatusByte: TButton;
    edtByteValue: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    lblPageNr: TLabel;
    Memo1: TMemo;
    rbtnSPIDev0: TRadioButton;
    rbtnSPIDev1: TRadioButton;
    spnedtAddress: TSpinEdit;
    spnedtByteValue: TSpinEdit;
    spnedtPageNr: TSpinEdit;
    procedure btnByteReadClick(Sender: TObject);
    procedure btnByteWriteClick(Sender: TObject);
    procedure btnDataInClick(Sender: TObject);
    procedure btnDataOutClick(Sender: TObject);
    procedure btnPageWriteClick(Sender: TObject);
    procedure btnReadStatusClick(Sender: TObject);
    procedure btnSendOnlyClick(Sender: TObject);
    procedure btnSendReceiveClick(Sender: TObject);
    procedure btnSetStatusByteClick(Sender: TObject);
    procedure btnSingleByteClick(Sender: TObject);
    procedure btnSetStatusSequentialClick(Sender: TObject);
    procedure btnSetStatusPageClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure rbtnSPIDev0Change(Sender: TObject);
    procedure rbtnSPIDev1Change(Sender: TObject);
  private
    { private declarations }
  public
    { public declarations }
  end;

const
  Instr_Read  = $03;
  Instr_Write = $02;
  Instr_RDSR  = $05;
  Instr_WRSR  = $01;


var
  Form1: TForm1;
  SRamAddr : Word;
  SRamDataByte : Byte;


implementation

{$R *.lfm}

{ TForm1 }

//**********************************************************
//* Procedure : Data in (Buffer Read)
//**********************************************************
procedure TForm1.btnDataInClick(Sender: TObject);
Var
  DataInValue : Integer;
  Din, cnt : Byte;
begin
  Din := 1;
  Cnt := 1;
  DataInValue := spi.DataIn(din,cnt);
  Memo1.lines.add('Data In : '+IntToStr(DataInValue));

end;

//**********************************************************
//* Procedure : Data Out (Buffer Write)
//**********************************************************
procedure TForm1.btnDataOutClick(Sender: TObject);    // Buffer Write
Var
  Result : Integer;
  Dout, cnt : Byte;
  OutArr : Array[0..4] of Byte;
begin
  OutArr[0] := Instr_Write ;
  OutArr[1] := 66;
  OutArr[2] := 34;
  OutArr[3] := 133;

  dout := 9;
  cnt := 4;
  Result := spi.DataOut(OutArr,cnt);
  // Result := spi.DataOut(dout,cnt);
end;

//**********************************************************
//* Procedure : Data Send And Receive
//**********************************************************
procedure TForm1.btnSendReceiveClick(Sender: TObject);
Var
  dout,din,len : Byte;
begin
  spi.TransferSync(dout,din,len);
end;

//**********************************************************
//* Procedure : Data Send Only
//**********************************************************
procedure TForm1.btnSendOnlyClick(Sender: TObject);
// Not clear now  what it does, if i have use  for that on SRAM Project
Var
  dout,len : Byte;
begin
  Memo1.lines.add('Proc : Send Data Only');

  dout := %00110011;  // This initializes only the first byte
 // len := 5;

  Memo1.lines.add('Data Out : '+IntToStr(Dout));
  Memo1.lines.add('Data Len : '+IntToStr(Len));
  spi.TransmitSync(dout,len);
end;

//**********************************************************
//* Procedure : Receive Single Byte
//**********************************************************
procedure TForm1.btnSingleByteClick(Sender: TObject);
// Not clear now  what it does, if i have use  for that on SRAM Project
Var
  ReceiveByte : Byte;
  dout : Byte;
begin
  ReceiveByte := spi.FastShift(dout);
  Memo1.lines.add('Data In : '+IntToStr(ReceiveByte));
end;

//**********************************************************
//* Procedure : Byte Write (seems to work correct)
//**********************************************************
procedure TForm1.btnByteWriteClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..3] of Byte;
begin
  Memo1.lines.add('Proc : Byte Write');

  OutArr[0] := %00000010;   //Instruction
  OutArr[1] := %00000000;   // MSB  A15-A8
  OutArr[2] := %00000000;   // LSB  A7-A0
  OutArr[3] := %01010101;   // Data to store
  ByteCount    := 4;
  spi.TransmitSync(OutArr, ByteCount);
  Memo1.lines.add('Stored Byte : '+IntToStr(OutArr[3]));
end;

//**********************************************************
//* Procedure : Byte Read (seems to work correct, but Byte_In is not correct)
//**********************************************************
procedure TForm1.btnByteReadClick(Sender: TObject);
Var
  OutArr : Array[0..2] of Byte;
  Byte_In  :  Byte; // Status
  ByteCount  :  Byte;    // Nr of Bytes to transmit
begin
  Memo1.lines.add('Proc : Byte Read');
  ByteCount    := 4;
  OutArr[0] := %00000011;   //Instruction
  OutArr[1] := %00000000;   // MSB  A15-A8
  OutArr[2] := %00000000;   // LSB  A7-A0

  spi.TransferSync(OutArr,Byte_In,ByteCount);
  Memo1.lines.add('Received Byte : '+IntToStr(Byte_In));
end;

//**********************************************************
//* Procedure : Page Write
//**********************************************************

procedure TForm1.btnPageWriteClick(Sender: TObject);
begin

end;

//**********************************************************
//* Procedure : Set Status To Byte Mode
//**********************************************************
procedure TForm1.btnSetStatusByteClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin
  Memo1.lines.add('Proc : Set to Byte Mode');

  OutArr[0] := %00000001;   //Instruction
  OutArr[1] := %00000001;   // Data to Status Register

  ByteCount    := 2;
  spi.TransmitSync(OutArr, ByteCount);

end;

//**********************************************************
//* Procedure : Set Status To Page Mode
//**********************************************************
procedure TForm1.btnSetStatusPageClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin
  Memo1.lines.add('Proc : Set to Page Mode');

  OutArr[0] := %00000001;   //Instruction
  OutArr[1] := %10000001;   // Data to Status Register

  ByteCount    := 2;
  spi.TransmitSync(OutArr, ByteCount);

end;

//**********************************************************
//* Procedure : Set Status To Sequential Mode
//**********************************************************
procedure TForm1.btnSetStatusSequentialClick(Sender: TObject);
Var
  ByteCount  :  Byte;    // Nr of Bytes to transmit
  OutArr : Array[0..1] of Byte;
begin
  Memo1.lines.add('Proc : Set to Sequential Mode');

  OutArr[0] := %00000001;   //Instruction
  OutArr[1] := %01000001;   // Data to Status Register

  ByteCount    := 2;
  spi.TransmitSync(OutArr, ByteCount);
end;

//**********************************************************
//* Procedure : Read Status Register
//**********************************************************
procedure TForm1.btnReadStatusClick(Sender: TObject);
//Sends one byte and receives one Byte (Status)
Var
  Instr_Out  :  Byte; // Instruction
  Status_In  :  Byte; // Status
  ByteCount  :  Byte;    // Nr of Bytes to transmit
begin
  Instr_Out := %00000101;
  Status_In := 0;
  ByteCount    := 2;
  spi.TransferSync(Instr_Out,Status_In,ByteCount);

  Case Status_In of
      1: Memo1.lines.add('Status : '+IntToStr(Status_In)+' Byte Mode');
     65: Memo1.lines.add('Status : '+IntToStr(Status_In)+' Sequential Mode');
    129: Memo1.lines.add('Status : '+IntToStr(Status_In)+' Page Mode');
    193: Memo1.lines.add('Status : '+IntToStr(Status_In)+' reserved');
    Else Memo1.lines.add('Status : '+IntToStr(Status_In)+' unknown'+' Len : '+IntToStr(ByteCount));
  end;

  // RAM really sends correct status, but Status_In do not show that  (shows 5, as Instr_Out)
end;

//**********************************************************
//* Procedure : Set SPI Device 0
//**********************************************************
procedure TForm1.rbtnSPIDev0Change(Sender: TObject);
begin
  If rbtnSPIDev0.checked then begin
    cSPI_DEVICE := '/dev/spidev0.0';  // Device 0, ChipSelect 0
    Memo1.lines.add('SPI Device : '+cSPI_DEVICE);
  end;
end;

//**********************************************************
//* Procedure : Set SPI Device 1
//**********************************************************
procedure TForm1.rbtnSPIDev1Change(Sender: TObject);
begin
  // Umschalten läuft noch nicht.
  //  cSPI_DEVICE  funktioniert nur als konstante
  If rbtnSPIDev1.checked then begin
    cSPI_DEVICE := '/dev/spidev0.1';  // Device 0, ChipSelect 1
    Memo1.lines.add('SPI Device : '+cSPI_DEVICE);
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  spi.Init();
  cSPI_DEVICE := '/dev/spidev0.0';  // Device 0, ChipSelect 0
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  spi.Close();
end;

end.

